#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <math.h>

#include <kohn_gif.h>

#define WIDTH 600
#define HEIGHT 600

struct _image_info
{
  int x0,y0;
  int radius;
  int circum;
  int width,height;
};

float calc_bit_len(struct _image_info *image_info, int radius, int len)
{
float segment;
int circum;

  circum=2*M_PI*radius;
  segment=2*M_PI*((float)len/(float)circum);

  return segment;
}

float calc_byte_len(struct _image_info *image_info, int radius, int len, uint8_t c)
{
float header,one,zero;
float segment;
int n;

  header=calc_bit_len(image_info, radius, len);
  one=header/2;
  zero=header/4;
  segment=0;

  for(n=0; n<8; n++)
  {
    segment=segment+(((c&0x80)==0)?zero:one)+zero;
    c=c<<1;
  }

  return segment;
}

float draw_block(uint8_t *image, struct _image_info *image_info, float start, int radius, int len)
{
float segment;
float o;
int circum;
int r,x,y;

  for (r=radius; r>radius-50; r--)
  {
    circum=2*M_PI*r;
    segment=2*M_PI*((float)len/(float)circum);

    //printf("r=%d\n", r);
    //printf("start=%f start+segment=%f circum=%d dr=%f\n", start, start+segment, circum, 6.18/circum);

    for (o=start; o<start+segment; o=o+(6.18/circum/4))
    {
      x=image_info->x0+(int)(r*cos(o));
      y=image_info->y0+(int)(r*sin(o));

      image[x+(y*image_info->width)]=1;
    }
  }

  return segment;
}

void draw_image(uint8_t *image, int width, int height)
{
struct _image_info image_info;
float o;
int x,y;

  image_info.x0=width/2;
  image_info.y0=height/2;
  image_info.width=width;
  image_info.height=height;

  image_info.radius=(width/2-10);
  image_info.circum=2*M_PI*image_info.radius;

  printf("circum=%d\n", image_info.circum);

  for (o=0; o<6.18; o=o+(6.18/image_info.circum))
  {
    x=image_info.x0+(int)(image_info.radius*cos(o));
    y=image_info.y0+(int)(image_info.radius*sin(o));

    image[x+(y*width)]=1;
  }

  for (o=0; o<6.18; o=o+(6.18/image_info.circum))
  {
    x=image_info.x0+(int)(8*cos(o));
    y=image_info.y0+(int)(8*sin(o));

    image[x+(y*width)]=1;
  }

#if 0
  draw_block(image, &image_info, 0, 200, 100);
  draw_block(image, &image_info, 3.14, image_info.radius, 100);
#endif

  FILE *in=fopen("test.txt","rb");
  int ch;
  int header=50;
  int radius=image_info.radius;
  float spacer;
  int n;
  if (in==0) { return; }

  o=0;

  while(1)
  {
    ch=getc(in);
    if (ch==EOF) { break; }
    printf("%c\n", ch);

    float byte_len=calc_byte_len(&image_info, radius, header, ch);
    //if (byte_len+o>6.18)
    if (byte_len+o>6.00)
    {
      if (o!=0)
      {
        o=0;
        radius-=50;
        if (radius<=100) break;
        byte_len=calc_byte_len(&image_info, radius, header, ch);
      }
    }

    if (o==0)
    {
      o=draw_block(image, &image_info, o, radius, header);
      spacer=o/4;
      o+=spacer;

      if (byte_len+o>6.18) { break; }
    }

    for(n=0; n<8; n++)
    {
      o+=draw_block(image, &image_info, o, radius, header/(((ch&0x80)==0)?4:2));
      o+=spacer;
      ch<<=1;
    }
  }

  fclose(in);
}

int main(int argc, char *argv[])
{
struct gif_info_t *gif_info;
uint8_t *image;
int n;

  image=(uint8_t *)malloc(WIDTH*HEIGHT);
  memset(image, 0, WIDTH*HEIGHT);

  draw_image(image, WIDTH, HEIGHT);

  gif_info=kgif_alloc_compress();
  gif_info->fp=fopen("out.gif","wb");

  if (gif_info->fp==0)
  {
    printf("Couldn't open output file.\n");
    exit(1);
  }

  gif_info->width=WIDTH;
  gif_info->height=HEIGHT;
  gif_info->colors=2;
  //gif_info->transparency_index=0;

  gif_info->color_map[0]=0xffffff;
  gif_info->color_map[1]=0x000000;

  kgif_write_header(gif_info);

  for(n=0; n<WIDTH*HEIGHT; n+=WIDTH)
  {
    kgif_compress_block(gif_info, image+n, WIDTH);
  }

  kgif_write_footer(gif_info);
  free(image);

  return 0;
}

